import numpy as np
import sys
import getopt
import random

X = 4
Y = 5

labelling_system = {
  'avg_precision': {
    'left': (0, 11),
    'center': (12, 19),
    'right': (20, 31)
  },
  'avg_redundant': {
    'left': (0, 11),
    'center1': (12, 19),
    'center2': (12, 19),
    'right': (20, 31)
  },
  'high_precision': {
    'far-left': (0, 5),
    'left': (6, 12),
    'center': (13, 18),
    'right': (19, 25),
    'far-right': (26, 31)
  },
  'high_redundant': {
    'far-left': (0, 5),
    'left': (6, 12),
    'center': (13, 18),
    'right1': (19, 25),
    'right2': (19, 25),
    'far-right': (26, 31)
  },
  'left_precision': {
    'far-left': (0, 4),
    'left': (5, 9),
    'near-left': (10, 13),
    'center': (14, 20),
    'right': (21, 31),
  },
  'right_precision': {
    'left': (0, 10),
    'center': (11, 17),
    'near-right': (18, 21),
    'right': (22, 26),
    'far-right': (27, 31)
  }
}

def process_labelling_system(_ls):
  processed_labelling_system = {}
  ordered_true_labels = []

  for system in _ls:
    prefix = ''.join(map(lambda x: x[0], system.split('_')))
    tmp = {}
    for label in _ls[system]:
      coords = _ls[system][label]
      # normalize
      tmp[prefix + '-' + label] = ((coords[0] / 31, coords[1] / 31), (coords[0] + coords[1]) / 2)
      ordered_true_labels.append((prefix + '-' + label, coords, (coords[0] + coords[1]) / 2))
    processed_labelling_system[prefix] = tmp

  labels_map = {}
  for i, label in enumerate(ordered_true_labels):
    labels_map[label[0]] = i
    labels_map[i] = label

  ordered_true_labels.sort(key=lambda x: x[2])

  return processed_labelling_system, ordered_true_labels, labels_map

def get_label(_label, _map):
  l = _map[_label]


def label_position(_p, _labelling_system):
  category = None

  for label in _labelling_system:
    if _p >= _labelling_system[label][0][0] and _p <= _labelling_system[label][0][1]:
      category = label

      if random.random() < 0.5:
        break

  return category

if __name__ == "__main__":
  LS = None
  cmd_line_opts, cmd_line_args = getopt.getopt(sys.argv[1:], 'l:')

  for opt in cmd_line_opts:
    if '-l' in opt:
      LS = opt[1]
  
  pls, otl, lm = process_labelling_system(labelling_system)
  # imgs range
  f = 0
  t = 0

  if LS is None:
    print("Ordered labels' true value:")
    for el in otl:
      print(f'{el[0]}: \t{el[2]}')

    print("Labels map:")
    print(lm)
  
  if LS is not None:
    print(f'Processing dataset with labelling system \'{LS}\'')
    ds = np.load('data/dsprites_ndarray_co1sh3sc6or40x32y32_64x64.npz', encoding='bytes')

    labels = []
    labels_used_x = set()
    labels_used_y = set()
    for element in ds['latents_values'][f:t]:
      label_x = label_position(element[X], pls[LS])
      label_y = label_position(element[Y], pls[LS])
      labels.append((lm[label_x], lm[label_y]))
      labels_used_x.add(lm[label_x])
      labels_used_y.add(lm[label_y])

    np.savez_compressed(f'data/dsprites_{LS}',
      imgs = ds['imgs'][f:t],
      latents_values = ds['latents_values'][f:t],
      labels = np.array(labels, dtype=np.int64),
      labels_used = (labels_used_x, labels_used_y),
      labels_info = {key: value for key, value in lm.items() if key in labels_used_x or key in labels_used_y}
    )
    print('Done')
