import numpy as np
import sys
import getopt
import random

X = 4
Y = 5

labelling_system = {
  'avg_precision': {
    'left': (0, 11),
    'center': (12, 19),
    'right': (20, 31)
  }
}

def process_labelling_system(_ls):
  processed_labelling_system = {}

  for system in _ls:
    prefix = ''.join(map(lambda x: x[0], system.split('_')))
    tmp = {}
    for label_x in _ls[system]:
      for label_y in _ls[system]:
        coords_x = _ls[system][label_x]
        coords_y = _ls[system][label_y]
        # normalize
        tmp[label_x + ':' + label_y] = (((coords_x[0] / 31, coords_x[1] / 31), (coords_x[0] + coords_x[1]) / 2),
          ((coords_y[0] / 31, coords_y[1] / 31), (coords_y[0] + coords_y[1]) / 2))
    processed_labelling_system[prefix] = tmp

  labels_map = {}
  for i, label_x in enumerate(_ls[system]):
    for y, label_y in enumerate(_ls[system]):
      t = i + y * len(_ls[system])
      labels_map[label_x + ':' + label_y] = t
      labels_map[t] = label_x + ':' + label_y

  return processed_labelling_system, labels_map

def get_label(_label, _map):
  l = _map[_label]


def label_position(_p, _labelling_system):
  category = None

  for label in _labelling_system:
    if _p[0] >= _labelling_system[label][0][0][0] and _p[0] <= _labelling_system[label][0][0][1] and \
      _p[1] >= _labelling_system[label][1][0][0] and _p[1] <= _labelling_system[label][1][0][1]:
      category = label

  return category

if __name__ == "__main__":
  LS = None
  cmd_line_opts, cmd_line_args = getopt.getopt(sys.argv[1:], 'l:')

  for opt in cmd_line_opts:
    if '-l' in opt:
      LS = opt[1]
  
  pls, lm = process_labelling_system(labelling_system)
  
  if LS is not None:
    print(f'Processing dataset with labelling system \'{LS}\'')
    ds = np.load('data/dsprites_ndarray_co1sh3sc6or40x32y32_64x64.npz', encoding='bytes')

    labels = []
    labels_used_x = set()
    labels_used_y = set()
    for element in ds['latents_values']:
      label = label_position((element[X], element[Y]), pls[LS])
      i_label = lm[label]
      labels.append((i_label, i_label))
      labels_used_x.add(i_label)
      labels_used_y.add(i_label)

    np.savez_compressed(f'data/dsprites_{LS}_mix',
      imgs = ds['imgs'],
      latents_values = ds['latents_values'],
      labels = np.array(labels, dtype=np.int64),
      labels_used = (labels_used_x, labels_used_y),
      labels_info = {key: pls[LS][value] for key, value in lm.items() if key in labels_used_x or key in labels_used_y}
    )
    print('Done')
